@@ -179,11 +179,22 @@ END
       URI::Amazon::APA 
       TheSchwartz
       Devel::CheckLib
+      File::Find::Rule
       Catalyst::Runtime
+      Regexp::Assemble
+      Catalyst::Controller::HTML::FormFu
+# https://metacpan.org/pod/DBIx::Class::Manual::Cookbook#Predefined-searches
+# http://ajct.info/2015/08/16/oauth-and-catalyst.html
+# http://stackoverflow.com/questions/23652166/how-to-generate-oauth-2-client-id-and-secret
+# https://bshaffer.github.io/oauth2-server-php-docs/grant-types/refresh-token/
+      Task::Catalyst::Tutorial
+      YAML::Syck
+      Catalyst::Model::Adaptor
 
    );
    my $install_catalyst=<<'END';
 
+
           o o    o .oPYo. ooooo    .oo o     o     o o    o .oPYo.
           8 8b   8 8        8     .P 8 8     8     8 8b   8 8    8
           8 8`b  8 `Yooo.   8    .P  8 8     8     8 8`b  8 8
@@ -199,7 +210,7 @@ END
                \__/_     ___ __ _| |_ __ _| |_   _ ___| |_
                /    \   / __/ _` | __/ _` | | | | / __| __|  Perl MVC
             _- |    |  | (_| (_| | || (_| | | |_| \__ \ |    framework
-       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|©
+       _ _-'   \____/   \___\__,_|\__\__,_|_|\__, |___/\__|c
      ((_)       ---\                         |___/
                     \
                      \\_          Web Framework
@@ -222,21 +233,96 @@ END
          print $install_catalyst;
          sleep 10;
       }
-      cmd_raw($handle,"sudo cpan $module",'__display__');
+      if ($module eq 'Regexp::Assemble') {
+         $handle->{_cmd_handle}->print($sudo.
+            'perl -MCPAN -e \'CPAN::Shell->force('.
+            "\"install\",\"$module\")\'"
+         )
+      } else {
+         $handle->{_cmd_handle}->print($sudo."cpan $module 2>&1");
+      }
+      my $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
+      my $error=0;my $force=0;my $tries=0;my $allout='';
+      while (1) {
+         my $done=eval {
+            local $SIG{ALRM} = sub { die "alarm\n" }; # \n required
+            alarm 120;
+            my $output=Net::FullAuto::FA_Core::fetch($handle);
+            $allout.=$output;
+            if ($output=~/$prompt/) {
+               if ($error) {
+                  $error=0;$force=1;
+                  $handle->{_cmd_handle}->print($sudo.
+                     'perl -MCPAN -e \'CPAN::Shell->force('.
+                     "\"install\",\"$module\")\'"
+                  )
+               } else {
+                  $output=~s/$prompt$//s;
+                  print $output;
+                  return 'done';
+               }
+            } elsif ($output=~/build the XS Stash module/) {
+               $handle->{_cmd_handle}->print('y');
+            } elsif ($output=~/use the XS Stash by default/) {
+               $handle->{_cmd_handle}->print('y');
+            }
+            if (!$force &&
+                  ((-1<index $allout,'[test_dynamic] Error 255') ||
+                  (-1<index $allout,'Connection reset by peer'))) {
+               $error=1;
+               $output=~s/$prompt//gs;
+            }
+            print $output;
+            return 'continue';
+         };
+         next if $done eq 'continue';
+         if ($done=~/^\d$/ && $done==1) {
+            my $output=Net::FullAuto::FA_Core::fetch($handle);
+            my $attempt='attempts';
+            $attempt='attempt' if $tries==0;
+            print "\n\n   FATAL ERROR!: Could not install CPAN Module",
+                  ":  $module\n\n",
+                  "                 --> $output\n",
+                  "                 after ",++$tries," $attempt\n\n";
+            &Net::FullAuto::FA_Core::cleanup;
+         } elsif ($@ && ++$tries<4) {
+            alarm(0);$allout='';
+            $handle->{_cmd_handle}->print("\003");
+            my $done=eval {
+               local $SIG{ALRM} = sub { die "alarm\n" }; # \n required
+               while (my $ln=$handle->{_cmd_handle}->get) {
+                  return 'done' if $ln=~/$prompt/s;
+               }
+            };
+            if ($@) {
+               my $attempt='attempts';
+               $attempt='attempt' if $tries==0;
+               print "\n\n   FATAL ERROR!: Could not install CPAN Module",
+                     ":  $module\n\n",
+                     "                 --> could not recover handle after \n",
+                     "                 ",++$tries," $attempt\n\n";
+               &Net::FullAuto::FA_Core::cleanup;
+            } elsif ($done) {
+               next
+            } else {
+               print "\n\n   FATAL ERROR!: Could not install CPAN Module",
+                     "                 $module\n",
+                     "                 - Unknown Error after ",
+                     --$tries," attempts\n\n";
+               &Net::FullAuto::FA_Core::cleanup;
+            }
+         } elsif ($tries>3) {
+            my $attempt='attempts';
+            print "\n\n   FATAL ERROR!: Could not install CPAN Module",
+                  ":  $module\n\n",
+                  "                 after ",++$tries," $attempt\n\n";
+            &Net::FullAuto::FA_Core::cleanup;
+         }
+         last if $done;
+      }
    }
    $show=<<END;
-########################################
 
-   INSTALLING Regexp::Assemble
-
-########################################
-END
-   print $show;
-   cmd_raw($handle,
-      'sudo perl -MCPAN -e \'CPAN::Shell->force('.
-      '"install","Regexp::Assemble")\'',
-      '__display__');
-   $show=<<END;
 ########################################
 
    INSTALLING Catalyst::Devel
@@ -244,9 +330,13 @@ END
 ########################################
 END
    print $show;
-   $handle->{_cmd_handle}->print( #'sudo cpan Catalyst::Devel');
-      'sudo perl -MCPAN -e \'CPAN::Shell->force('.
-      '"install","Regexp::Assemble")\'');
+   if ($^O eq 'cygwin') {
+      $handle->{_cmd_handle}->print($sudo.
+         'perl -MCPAN -e \'CPAN::Shell->notest('.
+         '"install","Catalyst::Devel")\'');
+   } else {
+      $handle->{_cmd_handle}->print($sudo.'cpan Catalyst::Devel');
+   }
    $prompt=substr($handle->{_cmd_handle}->prompt(),1,-1);
    while (1) {
       my $output.=Net::FullAuto::FA_Core::fetch($handle);
@@ -266,25 +356,6 @@ END
    $show=<<END;
 ########################################
 
-   INSTALLING Catalyst::Controller::HTML::FormFu
-
-########################################
-END
-   cmd_raw($handle,'sudo cpan Catalyst::Controller::HTML::FormFu',
-      '__display__');
-   $show=<<END;
-########################################
-
-   INSTALLING Task::Catalyst::Tutorial
-
-########################################
-END
-   print $show;
-   sleep 1;
-   cmd_raw($handle,'sudo cpan Task::Catalyst::Tutorial','__display__');
-   $show=<<END;
-########################################
-
    INSTALLING DBIx::Class::Schema::Loader
 
 ########################################
@@ -294,16 +365,6 @@ END
    $show=<<END;
 ########################################
 
-   INSTALLING YAML::Syck
-
-########################################
-END
-   print $show;
-   sleep 1;
-   cmd_raw($handle,'sudo cpan YAML::Syck','__display__');
-   $show=<<END;
-########################################
-
    INSTALLING Catalyst::Controller::REST
 
 ########################################
@@ -314,16 +375,6 @@ END
    $show=<<END;
 ########################################
 
-   INSTALLING Catalyst::Model::Adaptor
-
-########################################
-END
-   print $show;
-   sleep 1;
-   cmd_raw($handle,'sudo cpan Catalyst::Model::Adaptor','__display__');
-   $show=<<END;
-########################################
-
    INSTALLING Catalyst::View::JSON
 
 ########################################
